from typing import Dict, Tuple

# 双方向変換用辞書
KANA2PHONEME_BIJECTION: Dict[str, Tuple[str, ...]] = {
    # Vowels
    "ア": ("a",),
    "イ": ("i",),
    "ウ": ("u",),
    "エ": ("e",),
    "オ": ("o",),
    # Nasal
    "ン": ("N",),
    # K-row
    "カ": ("k", "a"),
    "キ": ("k", "i"),
    "ク": ("k", "u"),
    "ケ": ("k", "e"),
    "コ": ("k", "o"),
    # S-row
    "サ": ("s", "a"),
    "スィ": ("s", "i"),
    "ス": ("s", "u"),
    "セ": ("s", "e"),
    "ソ": ("s", "o"),
    # Sh-row
    "シャ": ("sh", "a"),
    "シ": ("sh", "i"),
    "シュ": ("sh", "u"),
    "シェ": ("sh", "e"),
    "ショ": ("sh", "o"),
    # T-row
    "タ": ("t", "a"),
    "ティ": ("t", "i"),
    "トゥ": ("t", "u"),
    "テ": ("t", "e"),
    "ト": ("t", "o"),
    # Ch-row
    "チャ": ("ch", "a"),
    "チ": ("ch", "i"),
    "チュ": ("ch", "u"),
    "チェ": ("ch", "e"),
    "チョ": ("ch", "o"),
    # Ts-row
    "ツァ": ("ts", "a"),
    "ツィ": ("ts", "i"),
    "ツ": ("ts", "u"),
    "ツェ": ("ts", "e"),
    "ツォ": ("ts", "o"),
    # N-row
    "ナ": ("n", "a"),
    "ニ": ("n", "i"),
    "ヌ": ("n", "u"),
    "ネ": ("n", "e"),
    "ノ": ("n", "o"),
    # H-row
    "ハ": ("h", "a"),
    "ヒ": ("h", "i"),
    # "フ": ("h", "u"),
    "ヘ": ("h", "e"),
    "ホ": ("h", "o"),
    # F-row
    "ファ": ("f", "a"),
    "フィ": ("f", "i"),
    "フ": ("f", "u"),
    "フェ": ("f", "e"),
    "フォ": ("f", "o"),
    # M-row
    "マ": ("m", "a"),
    "ミ": ("m", "i"),
    "ム": ("m", "u"),
    "メ": ("m", "e"),
    "モ": ("m", "o"),
    # Y-row
    "ヤ": ("y", "a"),
    "ユ": ("y", "u"),
    "イェ": ("y", "e"),
    "ヨ": ("y", "o"),
    # R-row
    "ラ": ("r", "a"),
    "リ": ("r", "i"),
    "ル": ("r", "u"),
    "レ": ("r", "e"),
    "ロ": ("r", "o"),
    # W-row
    "ワ": ("w", "a"),
    "ウィ": ("w", "i"),
    "ウェ": ("w", "e"),
    "ウォ": ("w", "o"),
    # G-row
    "ガ": ("g", "a"),
    "ギ": ("g", "i"),
    "グ": ("g", "u"),
    "ゲ": ("g", "e"),
    "ゴ": ("g", "o"),
    # Z-row
    "ザ": ("z", "a"),
    "ズィ": ("z", "i"),
    "ズ": ("z", "u"),
    "ゼ": ("z", "e"),
    "ゾ": ("z", "o"),
    # J-row
    "ジャ": ("j", "a"),
    "ジ": ("j", "i"),
    "ジュ": ("j", "u"),
    "ジェ": ("j", "e"),
    "ジョ": ("j", "o"),
    # D-row
    "ダ": ("d", "a"),
    "ディ": ("d", "i"),
    "ドゥ": ("d", "u"),
    "デ": ("d", "e"),
    "ド": ("d", "o"),
    # B-row
    "バ": ("b", "a"),
    "ビ": ("b", "i"),
    "ブ": ("b", "u"),
    "ベ": ("b", "e"),
    "ボ": ("b", "o"),
    # P-row
    "パ": ("p", "a"),
    "ピ": ("p", "i"),
    "プ": ("p", "u"),
    "ペ": ("p", "e"),
    "ポ": ("p", "o"),
    # V-row
    "ヴァ": ("v", "a"),
    "ヴィ": ("v", "i"),
    "ヴ": ("v", "u"),
    "ヴェ": ("v", "e"),
    "ヴォ": ("v", "o"),
    # --- ?y-row ---
    # Ky-row
    "キャ": ("ky", "a"),
    "キュ": ("ky", "u"),
    "キェ": ("ky", "e"),
    "キョ": ("ky", "o"),
    # Sy-row
    "スャ": ("sy", "a"),
    "スュ": ("sy", "u"),
    "スィェ": ("sy", "e"),
    "スョ": ("sy", "o"),
    # Ty-row
    "テャ": ("ty", "a"),
    "テュ": ("ty", "u"),
    "ティェ": ("ty", "e"),
    "テョ": ("ty", "o"),
    # Tsy-row
    "ツャ": ("tsy", "a"),
    "ツュ": ("tsy", "u"),
    "ツィェ": ("tsy", "e"),
    "ツョ": ("tsy", "o"),
    # Ny-row
    "ニャ": ("ny", "a"),
    "ニュ": ("ny", "u"),
    "ニェ": ("ny", "e"),
    "ニョ": ("ny", "o"),
    # Hy-row
    "ヒャ": ("hy", "a"),
    "ヒュ": ("hy", "u"),
    "ヒェ": ("hy", "e"),
    "ヒョ": ("hy", "o"),
    # Fy-row
    "フャ": ("fy", "a"),
    "フュ": ("fy", "u"),
    "フィェ": ("fy", "e"),
    "フョ": ("fy", "o"),
    # My-row
    "ミャ": ("my", "a"),
    "ミュ": ("my", "u"),
    "ミェ": ("my", "e"),
    "ミョ": ("my", "o"),
    # Ry-row
    "リャ": ("ry", "a"),
    "リュ": ("ry", "u"),
    "リェ": ("ry", "e"),
    "リョ": ("ry", "o"),
    # Vy-row
    "ヴャ": ("vy", "a"),
    "ヴュ": ("vy", "u"),
    "ヴィェ": ("vy", "e"),
    "ヴョ": ("vy", "o"),
    # Gy-row
    "ギャ": ("gy", "a"),
    "ギュ": ("gy", "u"),
    "ギェ": ("gy", "e"),
    "ギョ": ("gy", "o"),
    # By-row
    "ビャ": ("by", "a"),
    "ビュ": ("by", "u"),
    "ビェ": ("by", "e"),
    "ビョ": ("by", "o"),
    # Py-row
    "ピャ": ("py", "a"),
    "ピュ": ("py", "u"),
    "ピェ": ("py", "e"),
    "ピョ": ("py", "o"),
    # Dy-row
    "デャ": ("dy", "a"),
    "デュ": ("dy", "u"),
    "ディェ": ("dy", "e"),
    "デョ": ("dy", "o"),
    # Zy-row
    "ズャ": ("zy", "a"),
    "ズュ": ("zy", "u"),
    "ズィェ": ("zy", "e"),
    "ズョ": ("zy", "o"),
    # --- ?w-row ---
    # Kw-row
    "クァ": ("kw", "a"),
    # "クヮ": ("kw", "a"),
    "クィ": ("kw", "i"),
    "クェ": ("kw", "e"),
    "クォ": ("kw", "o"),
    # Sw-row
    "スァ": ("sw", "a"),
    # "スヮ": ("sw", "a"),
    "スゥィ": ("sw", "i"),
    "スェ": ("sw", "e"),
    "スォ": ("sw", "o"),
    # Nw-row
    "ヌァ": ("nw", "a"),
    # "ヌヮ": ("nw", "a"),
    "ヌィ": ("nw", "i"),
    "ヌェ": ("nw", "e"),
    "ヌォ": ("nw", "o"),
    # Pw-row
    "プァ": ("pw", "a"),
    # "プヮ": ("pw", "a"),
    "プィ": ("pw", "i"),
    "プェ": ("pw", "e"),
    "プォ": ("pw", "o"),
    # Mw-row
    "ムァ": ("mw", "a"),
    # "ムヮ": ("mw", "a"),
    "ムィ": ("mw", "i"),
    "ムェ": ("mw", "e"),
    "ムォ": ("mw", "o"),
    # Rw-row
    "ルァ": ("rw", "a"),
    # "ルヮ": ("rw", "a"),
    "ルィ": ("rw", "i"),
    "ルェ": ("rw", "e"),
    "ルォ": ("rw", "o"),
    # Gw-row
    "グァ": ("gw", "a"),
    # "グヮ": ("gw", "a"),
    "グィ": ("gw", "i"),
    "グェ": ("gw", "e"),
    "グォ": ("gw", "o"),
    # Zw-row
    "ズァ": ("zw", "a"),
    # "ズヮ": ("zw", "a"),
    "ズゥィ": ("zw", "i"),
    "ズェ": ("zw", "e"),
    "ズォ": ("zw", "o"),
    # Bw-row
    "ブァ": ("bw", "a"),
    # "ブヮ": ("bw", "a"),
    "ブィ": ("bw", "i"),
    "ブェ": ("bw", "e"),
    "ブォ": ("bw", "o"),
    # Shw-row
    "シュァ": ("shw", "a"),
    # "シュヮ": ("shw", "a"),
    # "シヮ": ("shw", "a"),
    "シュィ": ("shw", "i"),
    "シュェ": ("shw", "e"),
    "シュォ": ("shw", "o"),
    # Tw-row
    "トゥァ": ("tw", "a"),
    # "トヮ": ("tw", "a"),
    # "トァ": ("tw", "a"),
    "トゥィ": ("tw", "i"),
    # "トィ": ("tw", "i"),
    "トゥェ": ("tw", "e"),
    # "トェ": ("tw", "e"),
    "トゥォ": ("tw", "o"),
    # Chw-row
    "チュァ": ("chw", "a"),
    # "チュヮ": ("chw", "a"),
    # "チヮ": ("chw", "a"),
    "チュィ": ("chw", "i"),
    "チュェ": ("chw", "e"),
    "チュォ": ("chw", "o"),
    # Hw-row
    "ホゥァ": ("hw", "a"),
    # "ホゥヮ": ("hw", "a"),
    # "ホヮ": ("hw", "a"),
    "ホゥィ": ("hw", "i"),
    "ホゥェ": ("hw", "e"),
    "ホゥォ": ("hw", "o"),
    # Jw-row
    "ジュァ": ("jw", "a"),
    # "ジュヮ": ("jw", "a"),
    # "ジヮ": ("jw", "a"),
    "ジュィ": ("jw", "i"),
    "ジュェ": ("jw", "e"),
    "ジュォ": ("jw", "o"),
    # Dw-row
    "ドゥァ": ("dw", "a"),
    # "ドヮ": ("dw", "a"),
    "ドゥィ": ("dw", "i"),
    "ドゥェ": ("dw", "e"),
    "ドゥォ": ("dw", "o"),
    # 特殊な音
    "ッ": ("q",),  # 促音
    "ー": (":",),  # 長音符
}

# kana -> phoneme 用に追加する項目
FORWARD_ADDITIONAL: Dict[str, Tuple[str, ...]] = {
    # W-row
    "ヱ": ("w", "e"),
    "ヰ": ("w", "i"),
    "ヲ": ("o",),
    # ?wa variants
    "クヮ": ("kw", "a"),
    "スヮ": ("sw", "a"),
    "ヌヮ": ("nw", "a"),
    "プヮ": ("pw", "a"),
    "ムヮ": ("mw", "a"),
    "ルヮ": ("rw", "a"),
    "グヮ": ("gw", "a"),
    "ズヮ": ("zw", "a"),
    "ブヮ": ("bw", "a"),
    "シュヮ": ("shw", "a"),
    "シヮ": ("shw", "a"),
    "トヮ": ("tw", "a"),
    "トァ": ("tw", "a"),
    "トィ": ("tw", "i"),
    "トェ": ("tw", "e"),
    "トゥヮ": ("tw", "a"),
    "チヮ": ("chw", "a"),
    "ホヮ": ("hw", "a"),
    "ジヮ": ("jw", "a"),
    "ドヮ": ("dw", "a"),
    "ドァ": ("dw", "a"),
    "ドィ": ("dw", "i"),
    "ドェ": ("dw", "e"),
    # 小さい仮名（逆変換はできない）
    "ァ": ("a",),
    "ィ": ("i",),
    "ゥ": ("u",),
    "ェ": ("e",),
    "ォ": ("o",),
    "ャ": ("y", "a"),
    "ュ": ("y", "u"),
    "ョ": ("y", "o"),
    "ヮ": ("w", "a"),
    # 「ヶ」：通常は「カ」読み
    "ヶ": ("k", "a"),
}

# phoneme -> kana 用に追加する項目
BACKWARD_ADDITIONAL: Dict[Tuple[str, ...], str] = {
    ("h", "u"): "フ",
}
